@@ -1,18 +1,17 @@
-"""Utilities for matplotlib and seaborn."""
+"""Utilities for matplotlib and seaborn.
+
+`MatWrap.configure` is called upon importing this module, which enables default config.
+"""
 
-import json
-import warnings
-from argparse import _ArgumentGroup, Action, ArgumentParser
 from contextlib import AbstractContextManager
 from itertools import cycle, islice
 from types import ModuleType
-from typing import Any, List, Mapping, Optional, Tuple, Union
+from typing import Annotated, Any, Literal, Optional
 
-from corgy.types import KeyValueType
-from pkg_resources import resource_filename
+from corgy import Corgy
 
-_WRAPPED_NAMES = ["mpl", "plt", "sns"]
-__all__ = ["MatWrap", "Plot"] + _WRAPPED_NAMES
+_WRAPPED_NAMES = ("mpl", "plt", "sns")
+__all__ = ("MatWrap", "PlottingArgs", "Plot") + _WRAPPED_NAMES
 
 mpl: ModuleType
 plt: ModuleType
@@ -33,16 +32,16 @@ class MatWrap:
 
         # Do not import `matplotlib` or `seaborn`.
         from shinyutils.matwrap import MatWrap as mw
-        # Call before importing any packages that import matplotlib.
+        # Configure with `mw.configure` (refer to `configure` docs for details).
         mw.configure()
 
         fig = mw.plt().figure()
         ax = fig.add_subplot(111)  # `ax` can be used normally now
 
-        # Use class methods in `MatWrap` to access `matplotlib`/`seaborn` functions.
-        mw.mpl()  # returns `matplotlib` module
-        mw.plt()  # returns `matplotlib.pyplot` module
-        mw.sns()  # returns `seaborn` module
+        # Use class properties in `MatWrap` to access `matplotlib`/`seaborn` functions.
+        mw.mpl  # returns `matplotlib` module
+        mw.plt  # returns `matplotlib.pyplot` module
+        mw.sns  # returns `seaborn` module
 
         # You can also import the module names from `matwrap`
         from shinyutils.matwrap import mpl, plt, sns
@@ -51,24 +50,42 @@ class MatWrap:
         ...
     """
 
-    _rc_defaults_path = resource_filename("shinyutils", "data/mplcfg.json")
-    with open(_rc_defaults_path, "r", encoding="utf-8") as f:
-        _rc_defaults = json.load(f)
+    _rc_defaults: dict[str, Any] = {
+        "axes.grid": True,
+        "axes.grid.axis": "y",
+        "axes.grid.which": "major",
+        "axes.spines.bottom": False,
+        "axes.spines.left": False,
+        "axes.spines.right": False,
+        "axes.spines.top": False,
+        "backend": "pgf",
+        "figure.constrained_layout.use": True,
+        "legend.fancybox": False,
+        "legend.frameon": False,
+        "pgf.rcfonts": False,
+        "pgf.texsystem": "lualatex",
+        "savefig.format": "pdf",
+        "savefig.transparent": False,
+        "scatter.marker": ".",
+        "text.usetex": False,
+        "xtick.direction": "out",
+        "ytick.direction": "out",
+        "ytick.major.size": 0,
+    }
 
     _mpl = None
     _plt = None
     _sns = None
 
-    _args: Mapping[str, Any]
-    _mpl_default_rc: Mapping[str, Any]
+    _mpl_default_rc: dict[str, Any]
 
     @classmethod
     def configure(
         cls,
-        context: str = "paper",
-        style: str = "ticks",
+        context: Literal["paper", "poster", "notebook"] = "paper",
+        style: Literal["darkgrid", "whitegrid", "dark", "white", "ticks"] = "ticks",
         font: str = "Latin Modern Roman",
-        latex_pkgs: Optional[List[str]] = None,
+        latex_pkgs: Optional[list[str]] = None,
         backend: Optional[str] = None,
         **rc_extra,
     ):
@@ -98,10 +115,8 @@ class MatWrap:
             # pylint: disable=import-outside-toplevel
             try:
                 import matplotlib
-            except ImportError as e:
-                raise ImportError(
-                    f"{e}: install shinyutils[plotting] to use MatWrap"
-                ) from None
+            except ImportError:
+                raise ImportError("shinyutils.matwrap needs `matplotlib`") from None
 
             cls._mpl = matplotlib
             cls._mpl_default_rc = cls._mpl.rcParams.copy()
@@ -111,10 +126,8 @@ class MatWrap:
 
             try:
                 import seaborn
-            except ImportError as e:
-                raise ImportError(
-                    f"{e}: install shinyutils[plotting] to use MatWrap"
-                ) from None
+            except ImportError:
+                raise ImportError("shinyutils.matwrap needs `seaborn`") from None
 
             cls._plt = matplotlib.pyplot
             cls._sns = seaborn
@@ -128,15 +141,9 @@ class MatWrap:
             font_scale = 1
         cls._sns.set(context, style, cls.palette(), font_scale=font_scale, rc=rc)
 
-        cls._args = rc_extra.copy()
-        cls._args["context"] = context
-        cls._args["style"] = style
-        cls._args["font"] = font
-        cls._args["latex_pkgs"] = latex_pkgs
-
     def __new__(cls):
         raise NotImplementedError(
-            "MatWrap does not provide instances. Use the class methods."
+            "`MatWrap` does not provide instances. Use the class methods."
         )
 
     @classmethod
@@ -145,6 +152,7 @@ class MatWrap:
             cls.configure()
 
     @classmethod
+    @property
     def mpl(cls):
         """`matplotlib` module."""
         cls._ensure_conf()
@@ -152,6 +160,7 @@ class MatWrap:
         return cls._mpl
 
     @classmethod
+    @property
     def plt(cls):
         """`matplotlib.pyplot` module."""
         cls._ensure_conf()
@@ -159,6 +168,7 @@ class MatWrap:
         return cls._plt
 
     @classmethod
+    @property
     def sns(cls):
         """`seaborn` module."""
         cls._ensure_conf()
@@ -166,7 +176,7 @@ class MatWrap:
         return cls._sns
 
     @classmethod
-    def palette(cls, n=8) -> List[str]:
+    def palette(cls, n=8) -> list[str]:
         """Color universal design palette."""
         _base_palette = [
             "#000000",
@@ -183,115 +193,43 @@ class MatWrap:
 
         return list(islice(cycle(_base_palette), n))
 
-    @staticmethod
-    def set_size_tight(fig, size: Tuple[int, int]):
-        """Set the size of a matplotlib figure.
 
-        Args:
-            fig: Matplotlib `Figure` instance.
-            size: Tuple (width, height) in inches.
-        """
-        warnings.warn(
-            "constrained_layout is enabled by default: don't use tight_layout",
-            FutureWarning,
-        )
-        fig.set_size_inches(*size)
-        fig.tight_layout(pad=0, w_pad=0, h_pad=0)
+class PlottingArgs(Corgy):
+    """Plotting arguments that can be added to `ArgumentParser` instances.
 
-    @staticmethod
-    def add_parser_config_args(
-        base_parser: Union[ArgumentParser, _ArgumentGroup],
-        group_title: Optional[str] = "plotting options",
-    ) -> Union[ArgumentParser, _ArgumentGroup]:
-        """Add arguments for configuring plotting to a parser.
+    Usage::
 
-        Args:
-            base_parser: Argument parser or group to add arguments to.
-            group_title: Title to use for added options. If `None`, arguments will be
-                added to the base parser. Otherwise, options will be added to a group
-                with the given title. A new group will be created if `base_parser` is
-                not a group.
+        >>> arg_parser = ArgumentParser(add_help=False, formatter_class=Corgy)
+        >>> PlottingArgs.add_to_parser(arg_parser, name_prefix="plotting")
+        >>> arg_parser.print_help()
+        options:
+            --plotting-context str
+                seaborn plotting context ({'paper'/'notebook'/'talk'/'poster'}
+                default: 'paper')
+            --plotting-style str
+                seaborn plotting style
+                ({'white'/'dark'/'whitegrid'/'darkgrid'/'ticks'} default: 'ticks')
+            --plotting-font str
+                font for plots (default: 'Latin Modern Roman')
+            --plotting-backend str
+                matplotlib backend (default: 'pgf')
 
-        Example::
+    The class can also be used to create an argument group inside another `Corgy`
+    class::
 
-            >>> arg_parser = ArgumentParser(
-                    add_help=False, formatter_class=corgy.CorgyHelpFormatter
-                )
-            >>> MatWrap.add_parser_config_args(arg_parser)
-            >>> arg_parser.print_help()
-            plotting options:
-              --plotting-context str
-                  ({'paper'/'notebook'/'talk'/'poster'} default: 'paper')
-              --plotting-style str
-                  ({'white'/'dark'/'whitegrid'/'darkgrid'/'ticks'} default: 'ticks')
-              --plotting-font str
-                  (default: 'Latin Modern Roman')
-              --plotting-latex-pkgs [str [str ...]]
-                  (default: [])
-              --plotting-rc-extra [key=val [key=val ...]]
-                  (default: [])
-        """
+        class A(Corgy):
+            plotting: Annotated[PlottingArgs, "plotting arguments"]
+    """
 
-        class _ConfMatwrap(Action):
-            def __call__(self, parser, namespace, values, option_string=None):
-                _args = MatWrap._args
-                assert option_string.startswith("--plotting-")
-                option_name = option_string.split("--plotting-")[1].replace("-", "_")
-                if option_name == "rc_extra":
-                    _d = {}
-                    for k, v in values:
-                        try:
-                            _d[k] = int(v)
-                        except ValueError:
-                            try:
-                                _d[k] = float(v)
-                            except ValueError:
-                                _d[k] = v
-                    MatWrap.configure(**_args, **dict(values))
-                else:
-                    assert option_name in _args
-                    _args[option_name] = values
-                    MatWrap.configure(**_args)
-
-        if group_title is not None:
-            base_parser = base_parser.add_argument_group(group_title)
-
-        base_parser.add_argument(
-            "--plotting-context",
-            type=str,
-            choices=["paper", "notebook", "talk", "poster"],
-            default="paper",
-            action=_ConfMatwrap,
-        )
-        base_parser.add_argument(
-            "--plotting-style",
-            type=str,
-            choices=["white", "dark", "whitegrid", "darkgrid", "ticks"],
-            default="ticks",
-            action=_ConfMatwrap,
-        )
-        base_parser.add_argument(
-            "--plotting-font",
-            type=str,
-            default="Latin Modern Roman",
-            action=_ConfMatwrap,
-        )
-        base_parser.add_argument(
-            "--plotting-latex-pkgs",
-            type=str,
-            nargs="*",
-            default=[],
-            action=_ConfMatwrap,
-        )
-        base_parser.add_argument(
-            "--plotting-rc-extra",
-            type=KeyValueType(),
-            nargs="*",
-            default=[],
-            action=_ConfMatwrap,
-        )
-
-        return base_parser
+    context: Annotated[
+        Literal["paper", "notebook", "talk", "poster"], "seaborn plotting context"
+    ] = "paper"
+    style: Annotated[
+        Literal["white", "dark", "whitegrid", "darkgrid", "ticks"],
+        "seaborn plotting style",
+    ] = "ticks"
+    font: Annotated[str, "font for plots"] = "Latin Modern Roman"
+    backend: Annotated[str, "matplotlib backend"] = "pgf"
 
 
 class Plot(AbstractContextManager):
@@ -323,16 +261,16 @@ class Plot(AbstractContextManager):
         self,
         save_file: Optional[str] = None,
         title: Optional[str] = None,
-        sizexy: Optional[Tuple[int, int]] = None,
-        labelxy: Tuple[Optional[str], Optional[str]] = (None, None),
-        logxy: Tuple[bool, bool] = (False, False),
+        sizexy: Optional[tuple[int, int]] = None,
+        labelxy: tuple[Optional[str], Optional[str]] = (None, None),
+        logxy: tuple[bool, bool] = (False, False),
     ):
         self.save_file = save_file
         self.title = title
         self.sizexy = sizexy
         self.labelxy = labelxy
 
-        self.fig = MatWrap.plt().figure()
+        self.fig = MatWrap.plt.figure()  # type: ignore # mypy bug
         self.ax = self.fig.add_subplot(111)
 
         if logxy[0] is True:
@@ -360,7 +298,7 @@ class Plot(AbstractContextManager):
 
         if self.save_file is not None:
             self.fig.savefig(self.save_file)
-        MatWrap.plt().close(self.fig)
+        MatWrap.plt.close(self.fig)
 
 
 MatWrap.configure()
